#!/usr/local/bin/perl
#
# This script was sent to me (BdB) by Gary McTaggart (gmt@cis.ufl.edu).
# It finds the lines with time spent and sorts them.  The output is formatted
# such that when used from GNU Emacs M-x compile you can skip to the next line
# in the summary with C-x `

# $Id: dobprof,v 1.2 1994/12/24 07:00:03 gary Exp gary $

# Remove '#' from following lines to run bprof automatically when
# running this script.

#$exename = shift;
#`rm *.bprof`;
#`bprof $exename`;

foreach $prof_file (`ls -1 *.bprof`)
{
    chop( $prof_file );
    $file = $prof_file;
    $file =~ s/\.bprof.*//;
    open( IN, "<$prof_file" );
    while( <IN> )
    {
	if( /^\s+(\d+)\s+/ )
	{
	    chop;
	    push( @out, "$file:$. $1\n" );
	}
    }
    close( IN );
}

sub sort_sub
{
    local( $c, $d );
    $c = $a;
    $d = $b;
    $c =~ s/^\S+\s+(\S+)\s+.*/$1/;
    $d =~ s/^\S+\s+(\S+)\s+.*/$1/;
    return ( $d <=> $c );
}

# Sort from greatest to least by number of sample hits.
@out = sort sort_sub @out;

$out[0] =~ /^\S+\s+(\S+)/;
$max_count_length = length( $1 );
$max_count = $1;

for( $i = 0; $i < scalar( @out ); $i++ )
{
    $_ = $out[$i];
    /^(\S+)\s+(\S+)/;
    $max_line_specifier_length = length( $1 ) 
	if( $max_line_specifier_length < length( $1 ) );
    $total_samples += $2;
}

$max_graph_length = 80 - $max_line_specifier_length - $max_count_length - 20;

foreach $line ( @out )
{
    $line =~ /^(\S+)\s+(\S+)/;
    $line_specifier = $1;
    $count = $2;
    $percent = $count / $total_samples * 100;
    $cum_percent += $percent;
    $percent_string = sprintf( "%03.02f%%", $percent );
    $cum_percent_string = sprintf( "%03.02f%%", $cum_percent );
    print $line_specifier, &pad( $max_line_specifier_length - 
				  length( $line_specifier ) + 1, " " ),
	&pad( $max_count_length - length( $count ) + 1, " " ), $count,
	" ", &pad( 7 - length( $cum_percent_string ), " " ), $cum_percent_string,
	" ", &pad( 7 - length( $percent_string ), " " ), $percent_string,
	" ", &pad( $max_graph_length * $count / $max_count, "#" ), "\n";
				  
}

sub pad
{
    local( $len ) = shift;
    local( $char ) = shift;
    local( $i, $ret );
    for( $i = 0; $i < $len; $i++ )
    {
	$ret .= $char;
    }
    return $ret;
}
